
#include "hagutil.h"
#include "hagphysics2d.h"
#include <cmath> //for M_PI (mmmmm... pie)

/*
dt  timeslice

m   mass
mr  rotational-mass

p   position
x   x-position
y   y-position

v   velocity
vx  x-velocity
vy  y-velocity
vr  rotational-velocity

a   acceleration
ax  x-acceleration
ay  y-acceleration
ar  rotational-acceleration

f   force-magnitude (can be + or -)
fx  point of force-application x-component
fy  point of force-application y-component

comx  center-of-mass x-component
comy  center-of-mass y-component

*_angle   expressed in PI radians
*_normal  expressed in PI radians

*/



//position & velocity receive new values
void linear_acceleration(float &p, float &v, float a, float dt)
{
   p += v * dt  +  .5 * a * dt * dt;
   v += a * dt; //order of operations important here

//   v += a * dt;
//   p += v * dt;    
} 
// void linear_acceleration(float &position, float &velocity,  

                
             //Use AFTER linear_acceleration();
void speed_limit1d(float &v, float &x, float previous_x, 
                   float speed_limit, float dt)         //test for negative value!
{ 
   if(speed_limit < 0) speed_limit *= -1.0; // make sure speed_limit is positive                
                   
   if ( (v > 0) && (v > speed_limit) )
   {
      v = speed_limit; 
      x = previous_x + v * dt;
   }
   if ( (v < 0) && (v < -speed_limit) )
   {
      v = -speed_limit; 
      x = previous_x + v * dt;
   }
                                              
}  
// void speed_limit1d(float &v, float &x, float previous_x,


 
             //Use AFTER linear_acceleration();
void speed_limit2d(float &vx, float &vy, float &x, float &y, 
                   float previous_x, float previous_y, float speed_limit, float dt)
{ 
   float objectspeedsquared = vx * vx + vy * vy;
   if ( objectspeedsquared > (speed_limit * speed_limit) )
   {
      float objectspeed = sqrt(objectspeedsquared);
      
      if(speed_limit < 0) speed_limit *= -1.0; // make sure speed_limit is positive                

      vx = (speed_limit * vx) / objectspeed;
      vy = (speed_limit * vy) / objectspeed;
      x = previous_x +   vx * dt;
      y = previous_y +   vy * dt;
   }                                             
}  
// void speed_limit2d(float &vx, float &vy, float &x, float &y,


 
void gravity( float &ax, float &ay, 
              float x, float y, 
              float planet_x, float planet_y, float gravity_constant,
              bool linear_gravity)
{ 
   float dist = distance(x, y, planet_x, planet_y);

   float gravity_magnitude = 0.0;
    
   if( linear_gravity == true)            
       gravity_magnitude = gravity_constant / dist;
   else            
       gravity_magnitude = gravity_constant / (dist * dist);
                             
   float gravity_y = planet_y - y;
   float gravity_x = planet_x - x;
   normalize_vector(gravity_x, gravity_y);
   ax += gravity_magnitude * gravity_x;
   ay += gravity_magnitude * gravity_y;                                                                                        
}  
// void gravity_linear( float &ax, float &ay, 


                                                                 
//acceleration will receive a new value
void add_force(float &a, float m, float f)
{
    a += f / m;   //  (well, that was simple)           
}
// void add_force(float &a, float m, float f);



//all params are local coords
//acceleration and angular_acceleration will receive new values
void add_mounted_force(float &ax, float &ay, float &ar,
               float m, float mr, float body_angle,
               float fx, float fy, //relative to center of mass
               float f_angle, float f)
{
   ax += (f * cos(f_angle + body_angle)) / m;     
   ay += (f * sin(f_angle + body_angle)) / m;

   float tangental_force = sin(f_angle - atan2(fy,fx)); 
      //sin(f_angle - angle of force point relative to center of mass at body_angle == 0
   ar += (tangental_force * sqrt(fx * fx + fy * fy)) / mr; 
      //angular acceleration += tangental_force * radius / angular_mass    
}
// void add_mounted_force(float &ax, float &ay, float &ar,



//all params are world coordinates relative to center of mass
void add_force(float &ax, float &ay, float &ar,
               float m, float mr,
               float fx, float fy, //affected point on body
               float fx_direction, float fy_direction, //direction
               float f)
{
   float f_angle = atan2(fy_direction, fx_direction);          
   ax += (f * cos(f_angle)) / m;     
   ay += (f * sin(f_angle)) / m;
   
   float tangental_force = sin(f_angle - atan2(fy,fx)); 
   ar += (tangental_force * sqrt(fx * fx + fy * fy)) / mr; 
}
// void add_force(float &ax, float &ay, float &ar,
        
               

         //mass1,comx1,comy1 will receive new values
void add_mass(float &mass1, float &comx1, float &comy1,
              float mass2, float comx2, float comy2) //com = "center of mass"
{
   comx1 = (mass1 * comx1 + mass2 * comx2) / (mass1 + mass2);             
   comy1 = (mass1 * comy1 + mass2 * comy2) / (mass1 + mass2);
                             
   mass1 += mass2; // important that this be done last
}



/* NOT IMPLEMENTED:

         //mass1,angular_mass1,comx1,comy1 will receive new values
void add_mass(float &mass1, float &angular_mass1, float &comx1, float &comy1, 
                float mass2, float comx2, float comy2); //com = "center of mass"

             //Calculate velocity for 2 bodies post-collision 
             //mass1,angular_mass1,comx1,comy1 will receive new values
void elastic_collision( float &vx1, float &vy1, float mass1, //3 vars
             float &vx2, float &vy2, float mass2, //3 vars
             float impact_normal); //1 var

             //Calculate velocity and rotation for 2 bodies post-collision 
             //vx1,vy1,angular_v1,vx2,vy2,angular_v2 will receive new values
void elastic_collision( float &vx1, float &vy1, float &angular_v1, //3 vars
             float mass1, float angular_mass1, float comx1, float comy1, //4 vars
             float &vx2, float &vy2, float &angular_v2, //3 vars
             float mass2, float angular_mass2, float comx2, float comy2, //4 vars
             float impactx, float impacty, float impact_normal); //3 vars

*/



